 /*
  * Khoros: $Id$
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id$";
#endif

 /*
  * $Log$
  */ 

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1990, University of New Mexico.  All rights reserved.
 * 
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as too the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *----------------------------------------------------------------------
 */

#include "unmcopyright.h"	 /* Copyright 1990 by UNM */
#include "xvutils.h"	
#include "vsignal.h"	

static void   check_input_file();
static vsignal process_update();

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>                                                       <<<<
   >>>>	    	    file name: detect.c                       <<<<
   >>>>                                                       <<<<
   >>>>   These routines are used to detect whether a file    <<<<
   >>>>   has been changed.  In the case of 		      <<<<
   >>>>   standard input, when new input is detected then we  <<<<
   >>>>   call back the programmer's callback routine.  The   <<<<
   >>>>   programmer also specifies how often we are supposed <<<<
   >>>>   to check the file for modify time, but for a file   <<<<
   >>>>   descriptor we use the X toolkit to listen on the    <<<<
   >>>>   input port.					      <<<<
   >>>>                                                       <<<<
   >>>>                xvp_detect_file()		      <<<<              
   >>>>                xvp_delete_detect_file()               <<<<
   >>>>                check_input_file()                     <<<<
   >>>>                                                       <<<<
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

typedef struct _xvp_detect
{
	int	id, timeout_id;
	time_t  time;
	char	*filename;
	int	interval;
	int	(*routine)();
	caddr_t	clientData;

	int    force_callback;

	struct _xvp_detect *next, *prev;
} xvp_detect;

static xvp_detect *detect_list = NULL;




/************************************************************
*
*  MODULE NAME: xvp_detect_file
*
*      PURPOSE: Adds a handler to detect if a file has been changed.
*
*       OUTPUT: None
*
*    CALLED BY: Application programmer
*
*   WRITTEN BY: Mark Young
*
*
*************************************************************/


int xvp_detect_file(file, argtime, routine, clientData)

char	*file;
float	argtime;
int	(*routine)();
caddr_t clientData;
{
	struct stat   buf;
	xvp_detect    *entry;
	char	      *filename, error[512];


	/*
	 *  Get the last modified time of the file in question.
	 */
	if (!(filename = vfullpath(file, NULL, NULL)))
	{
	   sprintf(error,"Error!  Invalid filename found.");
	   xvf_error_wait(error, "xvp_detect_file", NULL);
	   return(FALSE);
	}

	if (!(entry = (xvp_detect *) calloc(1, sizeof(xvp_detect))))
	{
	   xvf_error_wait("Out of memory!", "xvp_detect_file", NULL);
	   return(FALSE);
	}

	if (stat(filename, &buf) != -1)
	   entry->time = buf.st_mtime;
	else
	   entry->time = time(NULL);

	entry->next = detect_list;
	entry->prev = NULL;
	if (detect_list != NULL) 
	   detect_list->prev = entry;

	detect_list = entry;

	/*
	 *  update the rest of detect structure info.  We also need
	 *  compute the update interval in terms of millaseconds instead
	 *  of seconds.
	 */
	entry->interval = argtime * 1000;
	entry->filename = filename;
	entry->routine  = routine;
	entry->clientData  = clientData;
	entry->id = (int) XtAppAddTimeOut(xvf_app_context, entry->interval,
				   check_input_file, (caddr_t) entry);

	entry->timeout_id = entry->id;
	signal(SIGCONT, process_update);
	kill(getppid(), SIGCONT);
	return(entry->id);
}

static vsignal process_update(signal, flags, context)

int     signal, flags;
struct  sigcontext *context;
{
	xvp_detect *entry = detect_list;


#ifdef DEBUG
	printf("SIGCONT called for process_update\n");
#endif
	while (entry != NULL)
	{
	   if (entry->filename != NULL && entry->force_callback == False)
	   {
	      /*
	       *  Set the forced callback so that all files are designated
	       *  touched.
	       */
	      entry->force_callback = True;

	      /*
	       *  Re-add check_input_file to the time out handler to call us
	       *  back immediately.
	       */
	      XtRemoveTimeOut((XtIntervalId) entry->timeout_id);
	      entry->timeout_id = (int) XtAppAddTimeOut(xvf_app_context, 0,
				check_input_file, (caddr_t) entry);
	   }
	   entry = entry->next;
	}
}



/************************************************************
*
*  MODULE NAME: xvp_delete_detect_file
*
*      PURPOSE: deletes a detect for a file specified by the
*		specified id.  We know whether to delete 
*
*       OUTPUT: None
*
*    CALLED BY: Application programmer
*
*   WRITTEN BY: Mark Young
*
*
*************************************************************/


xvp_delete_detect_file(id)

int	id;
{
	xvp_detect     *entry;


	if (detect_list == NULL)
	   return;

	entry = detect_list;
	while (entry->next != NULL && entry->id != id)
	   entry = entry->next;


	if (entry->id == id)
	{
	   if (entry->next == NULL && entry->prev == NULL)
	      detect_list = NULL;
	   else if (entry->prev == NULL)
	   {
	      entry->next->prev = NULL;
	      detect_list = entry->next;
	   }
	   else if (entry->next == NULL)
	   {
	      entry->prev->next = NULL;
	   }
	   else
	   {
	      entry->next->prev = entry->prev;
	      entry->prev->next = entry->next;
	   }

	   if (entry->filename != NULL)
	   {
	      free(entry->filename);
	      XtRemoveTimeOut((XtIntervalId) entry->timeout_id);
	   }
	   else
	   {
	      XtRemoveInput((XtInputId) entry->id);
	   }
	   free(entry);
	}
}



/************************************************************
*
*  MODULE NAME: check_input_file
*
*      PURPOSE: Checks to see if a file has changed.  If so 
*		we call the user's callback routine with the
*		id and client data.  If the user returns true
*		then we update the time otherwise we will check
*		the file again with the same time.
*
*       OUTPUT: None.  Calls the user's callback routine to perform
*		all necessary output.
*
*    CALLED BY: X toolkit thru the XtAppAddTimeOut mechanism
*
*   WRITTEN BY: Mark Young
*
*
*************************************************************/


static void  check_input_file(data, id)

caddr_t		data;
XtIntervalId	*id;
{
	struct stat buf, tmp_buf;
	int	    status, (*routine)();
	xvp_detect  *entry = (xvp_detect *) data;


	/*
	 *  Get the stat's for the entry filename, unless we have a force
	 *  callback.  If a force callback that simply do the callback no
	 *  matter what state we are in.
	 */
	if (stat(entry->filename, &buf) != -1 || entry->force_callback)
	{
	   /*
	    *  check to see if the file has been modified since the last
	    *  time we read it.
	    */
	   if (entry->time != buf.st_mtime || entry->force_callback)
	   {
	      /*
	       *  since the file has been modified call the user's callback
	       *  routine and update the read time.
	       */
	      routine    = entry->routine;

	      tmp_buf.st_mtime = entry->time;
	      entry->time = buf.st_mtime;
	      status = routine(entry->id, entry->filename, entry->clientData);
	      if (status == FALSE)
	         entry->time = tmp_buf.st_mtime;
	      else
	         entry->force_callback = False;
	   }
	}

	/*
	 *  Re-add check_input_file to the time out handler.
	 */
	entry->timeout_id = (int) XtAppAddTimeOut(xvf_app_context, 
					          entry->interval,
				                  check_input_file, data);
}

